use crate::zabbix::api::get_zabbix_authtoken;
use crate::zabbix::problems::DataMatrix;
use clap::App;
use nix::sys::inotify::{AddWatchFlags, InitFlags, Inotify, InotifyEvent};
use serde::{Deserialize, Serialize};
use serde_json::Error as JsonError;
use serde_json::Value as JsonValue;
use std::fs::File;
use std::io::Read;
use std::string::String;
use std::thread::sleep;
use std::time::Duration;

#[derive(Debug)]
pub enum ReloadFrequency {
    High = 50,
    Medium = 20,
    Low = 10,
}

#[derive(Debug, Clone)]
pub struct Context {
    pub cfg: Config,
    pub configfile: String,
    pub inotify: Inotify,
    pub datamatrix: DataMatrix,
    pub mode: String,
}

impl Context {
    pub async fn new() -> Self {
        let yaml = load_yaml!("cli.yml");
        let cli = App::from_yaml(yaml).get_matches();

        let configfile = cli.value_of("config").unwrap_or("config.json").to_string();

        let mut ctx = Context {
            cfg: Config::new(),
            configfile: configfile,
            inotify: Inotify::init(InitFlags::IN_NONBLOCK).unwrap(),
            datamatrix: DataMatrix::new(),
            mode: cli.value_of("mode").unwrap_or("draw").to_string(),
        };

        println!("Loading {configfile} file ...", configfile = ctx.configfile);
        ctx.cfg.load(&ctx.configfile).await;

        println!(
            "Adding inotify watch on {configfile} file ...",
            configfile = ctx.configfile
        );
        ctx.inotify
            .add_watch(ctx.configfile.as_str(), AddWatchFlags::IN_MODIFY)
            .unwrap();
        let tokenrefresh = get_zabbix_authtoken(&mut ctx.cfg);
        tokenrefresh.await;

        ctx
    }

    pub async fn hotreload(&mut self) {
        let events = match self.inotify.read_events() {
            Ok(ev) => ev,
            Err(_) => vec![],
        };
        if events.len() > 0 {
            println!("Reloading {cfg}", cfg = self.configfile);
            get_zabbix_authtoken(&mut self.cfg).await;
            self.cfg.load(self.configfile.as_str());
        }
    }
}

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct Config {
    pub server: String,
    pub username: String,
    pub password: String,
    pub authtoken: Option<String>,
    pub sloweffect: Option<bool>,
    pub refresh: Option<u64>,
    pub limit: Option<u64>,
}

impl<'a> Config {
    pub fn new() -> Self {
        Self {
            server: "https://zabbix.acme.com/api_jsonrpc.php".to_string(),
            username: "bob".to_string(),
            password: "password".to_string(),
            authtoken: Some("token".to_string()),
            sloweffect: Some(true),
            refresh: Some(5u64),
            limit: Some(20u64),
        }
    }

    async fn load(&'a mut self, configfile: &str) {
        let fileopen: Result<File, std::io::Error>;
        let filemeta = std::fs::metadata(configfile);
        let fileexists = match filemeta {
            Ok(_) => true,
            Err(_) => false,
        };

        if !fileexists {
            let _a = File::create(configfile);
        }
        fileopen = File::open(configfile);

        let mut file = match fileopen {
            Ok(f) => f,
            Err(err) => {
                panic!("{err}", err = err);
            }
        };
        let mut contents = String::new();
        file.read_to_string(&mut contents).unwrap();
        let parse: Result<JsonValue, JsonError> = serde_json::from_str(contents.as_str());
        *self = match parse {
            Ok(cfg) => {
                let tmpcfg = Config::new();
                Config {
<<<<<<< Updated upstream
                    server: cfg["server"].as_str().unwrap_or(tmpcfg.server.as_str()).to_string(),
                    username: cfg["username"].as_str().unwrap_or(tmpcfg.username.as_str()).to_string(),
                    password: cfg["password"].as_str().unwrap_or(tmpcfg.password.as_str()).to_string(),
                    authtoken: Some(cfg["authtoken"].as_str().unwrap_or(tmpcfg.authtoken.unwrap().as_str()).to_string()),
                    sloweffect: Some(cfg["sloweffect"].as_bool().unwrap_or(tmpcfg.sloweffect.unwrap())),
                    refresh: Some(cfg["refresh"].as_u64().unwrap_or(tmpcfg.refresh.unwrap())),
                    limit: Some(cfg["limit"].as_u64().unwrap_or(tmpcfg.limit.unwrap())),
=======
                    server: ncfg["server"]
                        .as_str()
                        .unwrap_or(&tmpcfg.server.clone())
                        .to_string(),
                    username: ncfg["username"]
                        .as_str()
                        .unwrap_or(&tmpcfg.username.clone())
                        .to_string(),
                    password: ncfg["password"]
                        .as_str()
                        .unwrap_or(&tmpcfg.password.clone())
                        .to_string(),
                    authtoken: Some(
                        ncfg["authtoken"]
                            .as_str()
                            .unwrap_or(self.authtoken.clone().unwrap().as_str())
                            .to_string(),
                    ),
                    sloweffect: Some(
                        ncfg["sloweffect"]
                            .as_bool()
                            .unwrap_or(tmpcfg.sloweffect.unwrap()),
                    ),
                    refresh: Some(ncfg["refresh"].as_u64().unwrap_or(tmpcfg.refresh.unwrap())),
                    limit: Some(ncfg["limit"].as_u64().unwrap_or(tmpcfg.limit.unwrap())),
>>>>>>> Stashed changes
                }
            }
            Err(err) => {
                println!("error occured: '{}'", err);
                Config::new()
            }
        };
<<<<<<< Updated upstream
        self.save(&configfile);
=======
        *self = newcfg;
        if !fileexists || errorreading {
            self.save(&configfile).await;
        }
>>>>>>> Stashed changes
    }

    async fn save(&self, configfile: &str) {
        let file = File::create(configfile).unwrap();
        serde_json::to_writer_pretty(file, &self).unwrap();
    }
}
